/*
 * This file is part of Deliantra, the Roguelike Realtime MMORPG.
 * 
 * Copyright (©) 2005,2006,2007,2008 Marc Alexander Lehmann / Robin Redeker / the Deliantra team
 * Copyright (©) 2002,2007 Mark Wedel & Crossfire Development Team
 * Copyright (©) 1992,2007 Frank Tore Johansen
 * 
 * Deliantra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * The authors can be reached via e-mail to <support@deliantra.net>
 */

#include <global.h>
#include <random_map.h>
#include <rproto.h>

/* where are there adjacent doors or walls? */
int
surround_check2 (char **layout, int i, int j, int Xsize, int Ysize)
{
  /* 1 = door or wall to left,
     2 = door or wall to right,
     4 = door or wall above
     8 = door or wall below */
  int surround_index = 0;

  if ((i > 0) && (layout[i - 1][j] == 'D' || layout[i - 1][j] == '#'))
    surround_index += 1;
  if ((i < Xsize - 1) && (layout[i + 1][j] == 'D' || layout[i + 1][j] == '#'))
    surround_index += 2;
  if ((j > 0) && (layout[i][j - 1] == 'D' || layout[i][j - 1] == '#'))
    surround_index += 4;
  if ((j < Ysize - 1) && (layout[i][j + 1] == 'D' && layout[i][j + 1] == '#'))
    surround_index += 8;
  return surround_index;
}

void
put_doors (maptile *the_map, char **maze, const char *doorstyle, random_map_params *RP)
{
  int i, j;
  maptile *vdoors;
  maptile *hdoors;
  char doorpath[1024];

  if (!strcmp (doorstyle, "none"))
    return;

  vdoors = find_style ("/styles/doorstyles", doorstyle, -1);

  if (vdoors)
    hdoors = vdoors;
  else
    {
      vdoors = find_style ("/styles/doorstyles/vdoors", doorstyle, -1);
      if (!vdoors)
        return;

      sprintf (doorpath, "/styles/doorstyles/hdoors%s", strrchr (vdoors->path, '/'));
      hdoors = find_style (doorpath, 0, -1);
      if (!hdoors)
        return;
    }

  for (i = 0; i < RP->Xsize; i++)
    for (j = 0; j < RP->Ysize; j++)
      {
        if (maze[i][j] == 'D')
          {
            int sindex;
            object *this_door, *new_door;

            sindex = surround_check2 (maze, i, j, RP->Xsize, RP->Ysize);
            if (sindex == 3)
              this_door = hdoors->pick_random_object ();
            else
              this_door = vdoors->pick_random_object ();

            if (!this_door)
              abort ();

            new_door = arch_to_object (this_door->arch);
            this_door->copy_to (new_door);
            new_door->x = i;
            new_door->y = j;
            insert_ob_in_map (new_door, the_map, NULL, 0);
          }
      }
}

